package org.ow2.orchestra.definition.activity;

import java.util.Date;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.orchestra.definition.BpelProcess;
import org.ow2.orchestra.facade.data.def.ProcessActivityDefinitionData;
import org.ow2.orchestra.facade.runtime.ActivityState;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;
import org.ow2.orchestra.pvm.internal.model.ObservableElementImpl;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.services.MessageCarrierRepository;
import org.ow2.orchestra.services.handlers.FinishedInstanceHandler;
import org.ow2.orchestra.services.itf.Replier;
import org.ow2.orchestra.util.EnvTool;

public class Process extends Scope {

  private static final Logger LOG = Logger.getLogger(Process.class.getName());

  public Process() {
    super();
  }

  @Override
  public void signal(final BpelExecution execution, final String signal, final Map<String, Object> parameters) {
    if (signal == null) {
      super.signal(execution, null, parameters);
      return;
    }
    // execute source to log end of scope ?
    this.afterRunned(execution);
    if (execution.isActive()) {
      final ObservableElementImpl parent = execution.getNode().getParent();
      if (parent == null) {
        if (Process.LOG.isLoggable(Level.FINE)) {
          Process.LOG.fine("End of bpel process execution");
        }
        if (execution.getParent() != null) {
          throw new IllegalStateException("Child execution reaches the end of process !");
        }
        if (!execution.searchWaitingExecutions().isEmpty()) {
          throw new IllegalStateException(
          "Execution reaches the end of process, but child executions are still waiting for a trigger !");
        }
        execution.removeJobs();
        execution.end();

      }
      // Call the FinishedInstanceHandler
      Process.endProcessInstance(execution);
    }
  }

  public static void endProcessInstance(final BpelExecution execution) {
    // close pending connections
    for (final Replier replier : execution.getRepliers()) {
      MessageCarrierRepository.throwMissingReply(replier.getReplierKey());
    }
    // Call the FinishedInstanceHandler
    final FinishedInstanceHandler handler = EnvTool.get(FinishedInstanceHandler.class, FinishedInstanceHandler.DEFAULT_KEY);
    if (handler != null) {
      handler.handleFinishedInstance(execution.getProcessInstanceUUID());

//      Environment.getFromCurrent(MessageSession.class).send(new HandleFinishedInstanceJob(execution.getProcessInstanceUUID()));
    }

  }

  @Override
  public ProcessActivityDefinitionData createDefinitionData(final BpelProcess bpelProcess) {
    return new ProcessActivityDefinitionData(bpelProcess.getProcessDefinitionData(), this.uuid, null, this.name, null);
  }

  @Override
  public void beforeRunning(final BpelExecution bpelExecution) {
    final ActivityInstanceUUID activityInstanceUUID = bpelExecution.getProcessInstanceUUID();
    bpelExecution.setStartedDate(new Date());
    bpelExecution.setActivityInstanceUUID(activityInstanceUUID);
    bpelExecution.setProcessInstanceState(ActivityState.RUNNING);

    EnvTool.getRecorder().recordInstanceStarted(bpelExecution.getProcessInstanceData());

  }

  @Override
  public void afterRunned(final BpelExecution bpelExecution) {
    bpelExecution.setProcessInstanceState(ActivityState.FINISHED);
    EnvTool.getRecorder().recordInstanceEnded(bpelExecution.getProcessInstanceData());
  }
}
