/**
 * Copyright (C) 2008 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.facade.jmx;

import java.lang.reflect.Proxy;
import java.util.List;
import java.util.Set;

import javax.xml.namespace.QName;

import org.ow2.orchestra.cluster.Server;
import org.ow2.orchestra.facade.Deployment;
import org.ow2.orchestra.facade.data.runtime.JobData;
import org.ow2.orchestra.facade.def.ActivityDefinition;
import org.ow2.orchestra.facade.def.ProcessDefinition;
import org.ow2.orchestra.facade.def.ProcessState;
import org.ow2.orchestra.facade.exception.ActivityInstanceNotFoundException;
import org.ow2.orchestra.facade.exception.ActivityNotFoundException;
import org.ow2.orchestra.facade.exception.InstanceNotFoundException;
import org.ow2.orchestra.facade.exception.JobNotFoundException;
import org.ow2.orchestra.facade.exception.ProcessNotFoundException;
import org.ow2.orchestra.facade.runtime.ActivityInstance;
import org.ow2.orchestra.facade.runtime.ActivityState;
import org.ow2.orchestra.facade.runtime.ProcessInstance;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;
import org.ow2.orchestra.jmx.ClassLoaderInterceptor;
import org.ow2.orchestra.jmx.RemoteDeployerImpl;
import org.ow2.orchestra.pvm.env.EnvironmentFactory;

public class RemoteDeployer implements RemoteDeployerMBean {

  private final RemoteDeployerMBean impl;

  public RemoteDeployer(final Server orchestraServer, final EnvironmentFactory environmentFactory) {
    final RemoteDeployerImpl remoteDeployer = new RemoteDeployerImpl(orchestraServer, environmentFactory);
    final Class<RemoteDeployerMBean> clazz = RemoteDeployerMBean.class;
    final ClassLoaderInterceptor clInterceptor = new ClassLoaderInterceptor(remoteDeployer);
    this.impl = (RemoteDeployerMBean) Proxy.newProxyInstance(RemoteDeployer.class.getClassLoader(), new Class[] {
      clazz
    }, clInterceptor);
  }

  public ProcessDefinition deploy(final Deployment deployment) {
    return this.impl.deploy(deployment);
  }

  public ProcessDefinition deployBar(final byte[] barFile) {
    return this.impl.deployBar(barFile);
  }

  public void initialize() {
    this.impl.initialize();
  }

  public void deployWebServices(final ProcessDefinitionUUID processDefinitionUUID) {
    this.impl.deployWebServices(processDefinitionUUID);
  }

  public void undeployWebServices(final ProcessDefinitionUUID processDefinitionUUID) {
    this.impl.undeployWebServices(processDefinitionUUID);
  }

  public List<ProcessDefinition> findProcessDefinitions() {
    return this.impl.findProcessDefinitions();
  }

  public boolean undeploy(final QName processQName) {
    return this.impl.undeploy(processQName);
  }

  public boolean undeploy(final ProcessDefinitionUUID processDefinitionUUID) {
    return this.impl.undeploy(processDefinitionUUID);
  }

  public boolean remove(final ProcessDefinitionUUID processDefinitionUUID) {
    return this.impl.remove(processDefinitionUUID);
  }

  public Set<ActivityInstance> findActivityInstances(final ProcessInstanceUUID processUUID) throws InstanceNotFoundException {
    return this.impl.findActivityInstances(processUUID);
  }

  public Set<ActivityInstance> findActivityInstances(final ProcessInstanceUUID processUUID, final ActivityState activityState)
    throws InstanceNotFoundException {
    return this.impl.findActivityInstances(processUUID, activityState);
  }

  public Set<ProcessInstance> findProcessInstances() {
    return this.impl.findProcessInstances();
  }

  public Set<ProcessInstance> findProcessInstances(final ProcessDefinitionUUID processDefinitionUUID)
    throws ProcessNotFoundException {
    return this.impl.findProcessInstances(processDefinitionUUID);
  }

  public Set<ProcessInstance> findProcessInstances(final ProcessDefinitionUUID processDefinitionUUID,
    final ActivityState processInstanceState) throws ProcessNotFoundException {
    return this.impl.findProcessInstances(processDefinitionUUID, processInstanceState);
  }

  public Set<ProcessInstance> findProcessInstances(final ActivityState processInstanceState) {
    return this.impl.findProcessInstances(processInstanceState);
  }

  public ActivityInstance getActivityInstance(final ActivityInstanceUUID activityInstanceUUID)
    throws ActivityInstanceNotFoundException {
    return this.impl.getActivityInstance(activityInstanceUUID);
  }

  public ProcessInstance getProcessInstance(final ProcessInstanceUUID processInstanceUUID) throws InstanceNotFoundException {
    return this.impl.getProcessInstance(processInstanceUUID);
  }

  public List<ProcessDefinition> findProcessDefinitions(final ProcessState state) {
    return this.impl.findProcessDefinitions(state);
  }

  public List<ProcessDefinition> findProcessDefinitions(final QName processQName, final ProcessState state) {
    return this.impl.findProcessDefinitions(processQName, state);
  }

  public List<ProcessDefinition> findProcessDefinitions(final QName processQName) {
    return this.impl.findProcessDefinitions(processQName);
  }

  public ActivityDefinition getActivityDefinition(final ActivityDefinitionUUID activityDefinitionUUID)
    throws ActivityNotFoundException {
    return this.impl.getActivityDefinition(activityDefinitionUUID);
  }

  public Set<ActivityDefinition> findActivityDefinitions(final ProcessDefinitionUUID processUUID)
    throws ProcessNotFoundException {
    return this.impl.findActivityDefinitions(processUUID);
  }

  public ProcessDefinition getProcessDefinition(final ProcessDefinitionUUID processUUID) throws ProcessNotFoundException {
    return this.impl.getProcessDefinition(processUUID);
  }

  public void exit(final ProcessInstanceUUID processInstanceUUID) throws InstanceNotFoundException {
    this.impl.exit(processInstanceUUID);
  }

  public void resume(final ProcessInstanceUUID processInstanceUUID) throws InstanceNotFoundException {
    this.impl.resume(processInstanceUUID);
  }

  public void suspend(final ProcessInstanceUUID processInstanceUUID) throws InstanceNotFoundException {
    this.impl.suspend(processInstanceUUID);
  }

  public boolean remove(final ProcessInstanceUUID processInstanceUUID) {
    return this.impl.remove(processInstanceUUID);
  }

  public List<JobData> findJobsWithException() {
    return this.impl.findJobsWithException();
  }

  public void retryJob(final JobData job, final int retries) throws JobNotFoundException {
    this.impl.retryJob(job, retries);
  }
}
