/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.facade.uuid;

import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.util.Misc;


public abstract class IdFactory {

  private IdFactory() {

  }

  public static ProcessInstanceUUID getNewInstanceUUID(final ProcessDefinitionUUID processUUID, final long instanceCounter) {
    final ProcessInstanceUUID processInstanceUUID =
      new ProcessInstanceUUID(processUUID + AbstractUUID.NUMBER_SEPARATOR + instanceCounter);
    return processInstanceUUID;
  }

  public static ProcessDefinitionUUID getNewProcessUUID(final String version) {
    final ProcessDefinitionUUID processDefinitionUUID = new ProcessDefinitionUUID(version);
    return processDefinitionUUID;
  }

  public static ActivityInstanceUUID getNewActivityInstanceUUID(final ProcessInstanceUUID processInstanceUUID) {
    final ActivityInstanceUUID activityUUID =
      new ActivityInstanceUUID(Misc.getUniqueId(processInstanceUUID + AbstractUUID.NUMBER_SEPARATOR));
    return activityUUID;
  }

  public static ActivityDefinitionUUID getNewActivityDefinitionUUID(final ProcessDefinitionUUID processUUID, final int activityCount) {
    final ActivityDefinitionUUID activityDefinitionUUID =
      new ActivityDefinitionUUID(processUUID, activityCount, ActivityType.UNKNOWN_ACTIVITY, null);
    return activityDefinitionUUID;
  }
}
