/**
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.jmx.commands;

import java.util.HashSet;
import java.util.Set;

import org.ow2.orchestra.facade.runtime.ActivityState;
import org.ow2.orchestra.facade.runtime.ProcessInstance;
import org.ow2.orchestra.facade.runtime.full.ProcessFullInstance;
import org.ow2.orchestra.facade.runtime.impl.ProcessInstanceImpl;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.pvm.env.Environment;
import org.ow2.orchestra.pvm.internal.cmd.Command;
import org.ow2.orchestra.services.Querier;
import org.ow2.orchestra.util.EnvTool;

public final class FindProcessInstancesCommand implements Command<Set<ProcessInstance>> {

  final ProcessDefinitionUUID processDefinitionUUID;
  final ActivityState processInstanceState;

  public FindProcessInstancesCommand(final ProcessDefinitionUUID processDefinitionUUID,
      final ActivityState processInstanceState) {
    this.processDefinitionUUID = processDefinitionUUID;
    this.processInstanceState = processInstanceState;
  }
  public FindProcessInstancesCommand(final ProcessDefinitionUUID processDefinitionUUID) {
    this.processDefinitionUUID = processDefinitionUUID;
    this.processInstanceState = null;
  }
  public FindProcessInstancesCommand(final ActivityState processInstanceState) {
    this.processDefinitionUUID = null;
    this.processInstanceState = processInstanceState;
  }
  public FindProcessInstancesCommand() {
    this.processDefinitionUUID = null;
    this.processInstanceState = null;
  }
  public Set<ProcessInstance> execute(final Environment environment) {
    final Set<ProcessInstance> result = new HashSet<ProcessInstance>();
    final Querier querier = EnvTool.getQuerier();
    Set<ProcessFullInstance> processFullInstances;
    if (this.processDefinitionUUID != null && this.processInstanceState != null) {
      processFullInstances = querier.findProcessInstances(this.processDefinitionUUID, this.processInstanceState);
    } else if (this.processDefinitionUUID != null) {
      processFullInstances = querier.findProcessInstances(this.processDefinitionUUID);
    } else if (this.processInstanceState != null) {
      processFullInstances = querier.findProcessInstances(this.processInstanceState);
    } else {
      processFullInstances = querier.findProcessInstances();
    }
    for (final ProcessFullInstance processFullInstance : processFullInstances) {
      result.add(new ProcessInstanceImpl(processFullInstance));
    }
    return result;
  }
}
