/**
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.osgi;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.logging.Level;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceRegistration;
import org.ow2.orchestra.util.Misc;


/**
 * @author Guillaume Porcher
 *
 */
public class OrchestraWSEngineActivator extends ExtensionActivator {

  /**
   * @author Guillaume Porcher
   *
   */
  private static final class EnvironmentProvider implements EnvironmentProviderService {

    private final URL envUrl;
    private final Bundle envBundle;

    EnvironmentProvider(final URL envUrl, final BundleContext bundleContext) {
      this.envUrl = envUrl;
      this.envBundle = bundleContext.getBundle();
    }
    public URL getEnvironmentURL() {
      return this.envUrl;
    }

    public Bundle getEnvironmentBundle() {
      return this.envBundle;
    }
  }

  // Default environment for deployed services
  private static final String DEFAULT_ENV_CONFIG_FILE = "conf/environment.xml";
  private static final String DEFAULT_ENV_CONFIG_RESOURCE = "environment.xml";

  private ServiceRegistration environmentRegistration;

  public OrchestraWSEngineActivator(final BundleContext context, final Class<?>...extensions) {
    super(context, extensions);
  }


  @Override
  public synchronized void start() throws Exception {
    super.start();
    URL envUrl = null;
    final File envFile = new File(OrchestraWSEngineActivator.DEFAULT_ENV_CONFIG_FILE);
    if (envFile.exists()) {
      try {
        envUrl = envFile.toURI().toURL();
      } catch (final MalformedURLException e) {
        Misc.unreachableStatement();
      }
    }
    if (envUrl == null) {
      envUrl = this.context.getBundle().getResource(OrchestraWSEngineActivator.DEFAULT_ENV_CONFIG_RESOURCE);
    }
    Misc.log(Level.INFO, "Environment will be taken from: %s", envUrl);

    this.environmentRegistration = this.context.registerService(
        EnvironmentProviderService.class.getName(),
        new EnvironmentProvider(envUrl, this.context), null);
  }

  @Override
  public synchronized void stop() throws Exception {
    if (this.environmentRegistration != null) {
      this.environmentRegistration.unregister();
      this.environmentRegistration = null;
    }
    super.stop();
  }

}
