/**
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.osgi.impl;

import java.util.List;

import org.ow2.orchestra.osgi.OrchestraExtensionService;

public class OrchestraExtensionClassLoader extends ClassLoader {

  private final List<OrchestraExtensionService> extensions;

  public OrchestraExtensionClassLoader(final List<OrchestraExtensionService> extensions) {
    super(OrchestraExtensionClassLoader.class.getClassLoader());
    this.extensions = extensions;
  }

  @Override
  protected Class< ? > findClass(final String name) throws ClassNotFoundException {
    for (final OrchestraExtensionService extensionService : this.extensions) {
      final Class< ? > extensionClass = extensionService.getExtension(name);
      if (extensionClass != null) {
        return extensionClass;
      }
    }
    throw new ClassNotFoundException(name);
  }
}