/**
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.runtime;

import java.util.HashSet;
import java.util.Set;

import org.ow2.orchestra.definition.element.Correlation;
import org.ow2.orchestra.definition.element.InboundMessageElement;
import org.ow2.orchestra.facade.exception.OrchestraRuntimeException;
import org.ow2.orchestra.services.ReceivingElement;


/**
 * @author Guillaume Porcher
 *
 */
public class WaitingExecution {

  public static interface WaitingExecutionIterator {
    WaitingExecution next();
  }

  protected long dbid;
  protected int dbversion;

  protected BpelExecution bpelExecution;
  protected Set<CorrelationSetRuntime> correlationSetRuntimes;
  protected InboundMessageElement inboundMessageElement;

  protected WaitingExecution() { }

  public WaitingExecution(
      final BpelExecution bpelExecution,
      final InboundMessageElement inboundMessageElement) {
    this.bpelExecution = bpelExecution;
    this.correlationSetRuntimes = new HashSet<CorrelationSetRuntime>();
    for (final ReceivingElement receivingElement : inboundMessageElement.getReceivingElements()) {
      for (final Correlation correlation : receivingElement.getCorrelations()) {
        final String correlationSetName = correlation.getCorrelationSet().getName();
        final CorrelationSetRuntime csRuntime = bpelExecution.getCorrelationSetByName(correlationSetName);
        if (csRuntime == null) {
          throw new OrchestraRuntimeException("CorrelationSetRuntime " + correlationSetName + " not found");
        }
        this.correlationSetRuntimes.add(csRuntime);
      }
    }
    this.inboundMessageElement = inboundMessageElement;
  }

  public BpelExecution getBpelExecution() {
    return this.bpelExecution;
  }
  public Set<CorrelationSetRuntime> getCorrelationSetRuntimes() {
    return this.correlationSetRuntimes;
  }
  public InboundMessageElement getInboundMessageElement() {
    return this.inboundMessageElement;
  }

  @Override
  public String toString() {
    return "WaitingExecution[" + this.dbid + "]";
  }
}
