package org.ow2.orchestra.services;

import java.io.Serializable;

import javax.xml.namespace.QName;

import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;

public class OperationKey implements Serializable {

  private static final long serialVersionUID = 7629200185946199717L;

  /**
   * represent the key of each 'exchange' activity like receive, reply, invoke,
   * pick...
   *
   */
  protected ProcessDefinitionUUID processUUID;

  protected QName portTypeQName;

  protected String operationName;

  /** For persistence * */
  protected OperationKey() {
  }

  public OperationKey(final ProcessDefinitionUUID processUUID, final QName portTypeQName,
      final String operationName) {
    this.processUUID = processUUID;
    this.portTypeQName = portTypeQName;
    this.operationName = operationName;
  }

  @Override
  public boolean equals(final Object o) {
    if (!(o instanceof OperationKey)) {
      return false;
    }
    return this.toString().equals(o.toString());
  }

  @Override
  public int hashCode() {
    return this.toString().hashCode();
  }

  public String getOperationName() {
    return this.operationName;
  }

  public QName getPortTypeQName() {
    return this.portTypeQName;
  }

  public ProcessDefinitionUUID getProcessUUID() {
    return this.processUUID;
  }

  @Override
  public String toString() {
    return "[" + this.processUUID + " " + this.portTypeQName + " " + this.operationName + "]";
  }

  public static OperationKey fromString(final String value) {
    // remove []
    final String s = value.substring(1, value.length() - 1);
    // split
    final String[] vals = s.split(" ");
    // create operationKey
    return new OperationKey(new ProcessDefinitionUUID(vals[0]), QName.valueOf(vals[1]), vals[2]);
  }
}
